@@ -26,10 +26,12 @@
 
 import binascii
 import struct
+from typing import Callable, Optional, Union
 
 import txaio
 
 from autobahn import util
+from autobahn.wamp.interfaces import IEd25519Key, ISession
 from autobahn.wamp.types import Challenge
 
 __all__ = [
@@ -140,8 +142,9 @@ class _SSHPacketReader:
         return self.get_bytes(self.get_uint32())
 
 
-def _makepad(size):
-    return ''.join(chr(x) for x in range(1, size + 1))
+def _makepad(size: int) -> bytes:
+    assert 0 <= size < 255
+    return b''.join(x.to_bytes(1, byteorder='big') for x in range(1, size + 1))
 
 
 def _read_ssh_ed25519_privkey(keydata):
@@ -353,32 +356,85 @@ def _verify_signify_ed25519_signature(pubkey_file, signature_file, message):
 
 if HAS_CRYPTOSIGN:
 
-    @util.public
-    class SigningKey(object):
+    def format_challenge(challenge: Challenge, channel_id_raw: Optional[bytes], channel_id_type: Optional[str]) -> bytes:
         """
-        A cryptosign private key for signing, and hence usable for authentication or a
-        public key usable for verification (but can't be used for signing).
+        Format the challenge based on provided parameters
+
+        :param challenge: The WAMP-cryptosign challenge object for which a signature should be computed.
+        :param channel_id_raw: The channel ID when channel_id_type is 'tls-unique'.
+        :param channel_id_type: The type of the channel id, currently handles 'tls-unique' and
+            ignores otherwise.
         """
+        if not isinstance(challenge, Challenge):
+            raise Exception(
+                "challenge must be instance of autobahn.wamp.types.Challenge, not {}".format(type(challenge)))
 
-        def __init__(self, key, comment=None):
-            """
+        if 'challenge' not in challenge.extra:
+            raise Exception("missing challenge value in challenge.extra")
 
-            :param key: A Ed25519 private signing key or a Ed25519 public verification key.
-            :type key: instance of nacl.signing.VerifyKey or instance of nacl.signing.SigningKey
-            """
-            if not (isinstance(key, signing.VerifyKey) or isinstance(key, signing.SigningKey)):
-                raise Exception("invalid type {} for key".format(type(key)))
+        # the challenge sent by the router (a 32 bytes random value)
+        challenge_hex = challenge.extra['challenge']
 
-            if not (comment is None or type(comment) == str):
-                raise Exception("invalid type {} for comment".format(type(comment)))
+        if type(challenge_hex) != str:
+            raise Exception("invalid type {} for challenge (expected a hex string)".format(type(challenge_hex)))
 
-            self._key = key
-            self._comment = comment
-            self._can_sign = isinstance(key, signing.SigningKey)
+        if len(challenge_hex) != 64:
+            raise Exception("unexpected challenge (hex) length: was {}, but expected 64".format(len(challenge_hex)))
 
-        def __str__(self):
-            comment = '"{}"'.format(self.comment()) if self.comment() else None
-            return 'Key(can_sign={}, comment={}, public_key={})'.format(self.can_sign(), comment, self.public_key())
+        # the challenge for WAMP-cryptosign is a 32 bytes random value in Hex encoding (that is, a unicode string)
+        challenge_raw = binascii.a2b_hex(challenge_hex)
+
+        if channel_id_type == 'tls-unique':
+            assert len(
+                channel_id_raw) == 32, 'unexpected TLS transport channel ID length (was {}, but expected 32)'.format(
+                len(channel_id_raw))
+
+            # with TLS channel binding of type "tls-unique", the message to be signed by the client actually
+            # is the XOR of the challenge and the TLS channel ID
+            data = util.xor(challenge_raw, channel_id_raw)
+        elif channel_id_type is None:
+            # when no channel binding was requested, the message to be signed by the client is the challenge only
+            data = challenge_raw
+        else:
+            assert False, 'invalid channel_id_type "{}"'.format(channel_id_type)
+
+        return data
+
+    def sign_challenge(data: bytes, signer_func: Callable) -> bytes:
+        """
+        Sign the provided data using the provided signer.
+
+        :param data: challenge to sign
+        :param signer_func: The callable function to use for signing
+        :returns: A Deferred/Future that resolves to the computed signature.
+        :rtype: str
+        """
+        # a raw byte string is signed, and the signature is also a raw byte string
+        d1 = signer_func(data)
+
+        # asyncio lacks callback chaining (and we cannot use co-routines, since we want
+        # to support older Pythons), hence we need d2
+        d2 = txaio.create_future()
+
+        def process(signature_raw):
+            # convert the raw signature into a hex encode value (unicode string)
+            signature_hex = binascii.b2a_hex(signature_raw).decode('ascii')
+
+            # we return the concatenation of the signature and the message signed (96 bytes)
+            data_hex = binascii.b2a_hex(data).decode('ascii')
+
+            sig = signature_hex + data_hex
+            txaio.resolve(d2, sig)
+
+        txaio.add_callbacks(d1, process, None)
+
+        return d2
+
+    class SigningKeyBase(object):
+
+        def __init__(self, signer, can_sign: bool) -> None:
+            self._can_sign = can_sign
+            self._key = signer
 
         @util.public
         def can_sign(self):
@@ -391,35 +447,28 @@ if HAS_CRYPTOSIGN:
             return self._can_sign
 
         @util.public
-        def comment(self):
+        def sign_challenge(self, session: ISession, challenge: Challenge, channel_id_type: Optional[str] = None) -> bytes:
             """
-            Get the key comment (if any).
+            Sign WAMP-cryptosign challenge.
 
-            :returns: The comment (if any) from the key.
-            :rtype: str or None
+            :param session: The authenticating WAMP session.
+            :param challenge: The WAMP-cryptosign challenge object for which a signature should be computed.
+            :returns: A Deferred/Future that resolves to the computed signature.
             """
-            return self._comment
+            # get the TLS channel ID of the underlying TLS connection
+            channel_id_type = 'tls-unique'
+            if channel_id_type in session._transport.transport_details.channel_id:
+                channel_id = session._transport.transport_details.channel_id.get(channel_id_type, None)
+            else:
+                channel_id_type = None
+                channel_id = None
+
+            data = format_challenge(challenge, channel_id, channel_id_type)
+
+            return sign_challenge(data, self.sign)
 
         @util.public
-        def public_key(self, binary=False):
-            """
-            Returns the public key part of a signing key or the (public) verification key.
-
-            :returns: The public key in Hex encoding.
-            :rtype: str or None
-            """
-            if isinstance(self._key, signing.SigningKey):
-                key = self._key.verify_key
-            else:
-                key = self._key
-
-            if binary:
-                return key.encode()
-            else:
-                return key.encode(encoder=encoding.HexEncoder).decode('ascii')
-
-        @util.public
-        def sign(self, data):
+        def sign(self, data: bytes) -> bytes:
             """
             Sign some data.
 
@@ -439,81 +488,71 @@ if HAS_CRYPTOSIGN:
             sig = self._key.sign(data)
 
             # we only return the actual signature! if we return "sig",
-            # it get coerced into the concatenation of message + signature
+            # it gets coerced into the concatenation of message + signature
             # not sure which order, but we don't want that. we only want
             # the signature
             return txaio.create_future_success(sig.signature)
 
+    @util.public
+    class SigningKey(SigningKeyBase):
+        """
+        A cryptosign private key for signing, and hence usable for authentication or a
+        public key usable for verification (but can't be used for signing).
+        """
+
+        def __init__(self, key, comment=None):
+            """
+
+            :param key: A Ed25519 private signing key or a Ed25519 public verification key.
+            :type key: instance of nacl.signing.VerifyKey or instance of nacl.signing.SigningKey
+            """
+            if not (isinstance(key, signing.VerifyKey) or isinstance(key, signing.SigningKey)):
+                raise Exception("invalid type {} for key".format(type(key)))
+
+            if not (comment is None or type(comment) == str):
+                raise Exception("invalid type {} for comment".format(type(comment)))
+
+            self._key = key
+            self._comment = comment
+            can_sign = isinstance(key, signing.SigningKey)
+
+            super().__init__(key, can_sign)
+
+        def __str__(self):
+            comment = '"{}"'.format(self.comment()) if self.comment() else None
+            return 'Key(can_sign={}, comment={}, public_key={})'.format(self.can_sign(), comment, self.public_key())
+
         @util.public
-        def sign_challenge(self, session, challenge, channel_id_type='tls-unique'):
+        def comment(self):
             """
-            Sign WAMP-cryptosign challenge.
+            Get the key comment (if any).
 
-            :param session: The authenticating WAMP session.
-            :type session: :class:`autobahn.wamp.protocol.ApplicationSession`
-
-            :param challenge: The WAMP-cryptosign challenge object for which a signature should be computed.
-            :type challenge: instance of autobahn.wamp.types.Challenge
-
-            :returns: A Deferred/Future that resolves to the computed signature.
-            :rtype: str
+            :returns: The comment (if any) from the key.
+            :rtype: str or None
             """
-            if not isinstance(challenge, Challenge):
-                raise Exception("challenge must be instance of autobahn.wamp.types.Challenge, not {}".format(type(challenge)))
+            return self._comment
 
-            if 'challenge' not in challenge.extra:
-                raise Exception("missing challenge value in challenge.extra")
+        @util.public
+        def public_key(self, binary: bool = False) -> Union[str, bytes]:
+            """
+            Returns the public key part of a signing key or the (public) verification key.
 
-            # the challenge sent by the router (a 32 bytes random value)
-            challenge_hex = challenge.extra['challenge']
-
-            if type(challenge_hex) != str:
-                raise Exception("invalid type {} for challenge (expected a hex string)".format(type(challenge_hex)))
-
-            if len(challenge_hex) != 64:
-                raise Exception("unexpected challenge (hex) length: was {}, but expected 64".format(len(challenge_hex)))
-
-            # the challenge for WAMP-cryptosign is a 32 bytes random value in Hex encoding (that is, a unicode string)
-            challenge_raw = binascii.a2b_hex(challenge_hex)
-
-            if channel_id_type == 'tls-unique':
-                # get the TLS channel ID of the underlying TLS connection
-                channel_id_raw = session._transport.get_channel_id()
-                assert len(channel_id_raw) == 32, 'unexpected TLS transport channel ID length (was {}, but expected 32)'.format(len(channel_id_raw))
-
-                # with TLS channel binding of type "tls-unique", the message to be signed by the client actually
-                # is the XOR of the challenge and the TLS channel ID
-                data = util.xor(challenge_raw, channel_id_raw)
-            elif channel_id_type is None:
-                # when no channel binding was requested, the message to be signed by the client is the challenge only
-                data = challenge_raw
+            :returns: The public key in Hex encoding.
+            :rtype: str or None
+            """
+            if isinstance(self._key, signing.SigningKey):
+                key = self._key.verify_key
             else:
-                assert False, 'invalid channel_id_type "{}"'.format(channel_id_type)
+                key = self._key
 
-            # a raw byte string is signed, and the signature is also a raw byte string
-            d1 = self.sign(data)
-
-            # asyncio lacks callback chaining (and we cannot use co-routines, since we want
-            # to support older Pythons), hence we need d2
-            d2 = txaio.create_future()
-
-            def process(signature_raw):
-                # convert the raw signature into a hex encode value (unicode string)
-                signature_hex = binascii.b2a_hex(signature_raw).decode('ascii')
-
-                # we return the concatenation of the signature and the message signed (96 bytes)
-                data_hex = binascii.b2a_hex(data).decode('ascii')
-
-                sig = signature_hex + data_hex
-                txaio.resolve(d2, sig)
-
-            txaio.add_callbacks(d1, process, None)
-
-            return d2
+            if binary:
+                return key.encode()
+            else:
+                return key.encode(encoder=encoding.HexEncoder).decode('ascii')
 
         @util.public
         @classmethod
-        def from_key_bytes(cls, keydata, comment=None):
+        def from_key_bytes(cls, keydata: str, comment: Optional[str] = None) -> 'SigningKey':
             if not (comment is None or type(comment) == str):
                 raise ValueError("invalid type {} for comment".format(type(comment)))
 
@@ -526,8 +565,9 @@ if HAS_CRYPTOSIGN:
             key = signing.SigningKey(keydata)
             return cls(key, comment)
 
+        @util.public
         @classmethod
-        def from_raw_key(cls, filename, comment=None):
+        def from_raw_key(cls, filename: str, comment: Optional[str] = None) -> 'SigningKey':
             """
             Load an Ed25519 (private) signing key (actually, the seed for the key) from a raw file of 32 bytes length.
             This can be any random byte sequence, such as generated from Python code like
@@ -556,7 +596,7 @@ if HAS_CRYPTOSIGN:
 
         @util.public
         @classmethod
-        def from_ssh_key(cls, filename):
+        def from_ssh_key(cls, filename: str) -> 'SigningKey':
             """
             Load an Ed25519 key from a SSH key file. The key file can be a (private) signing
             key (from a SSH private key file) or a (public) verification key (from a SSH
@@ -569,7 +609,7 @@ if HAS_CRYPTOSIGN:
 
         @util.public
         @classmethod
-        def from_ssh_data(cls, keydata):
+        def from_ssh_data(cls, keydata: str) -> 'SigningKey':
             """
             Load an Ed25519 key from SSH key file. The key file can be a (private) signing
             key (from a SSH private key file) or a (public) verification key (from a SSH
@@ -587,26 +627,4 @@ if HAS_CRYPTOSIGN:
 
             return cls(key, comment)
 
-if __name__ == '__main__':
-    import sys
-    if not HAS_CRYPTOSIGN:
-        print('NaCl library must be installed for this to function.', file=sys.stderr)
-        sys.exit(1)
-
-    from optparse import OptionParser
-
-    parser = OptionParser()
-    parser.add_option('-f', '--file', dest='keyfile',
-                      help='file containing ssh key')
-    parser.add_option('-p', action='store_true', dest='printpub', default=False,
-                      help='print public key information')
-
-    options, args = parser.parse_args()
-
-    if not options.printpub:
-        print("Print public key must be specified as it's the only option.")
-        parser.print_usage()
-        sys.exit(1)
-
-    key = SigningKey.from_ssh_key(options.keyfile)
-    print(key.public_key())
+    IEd25519Key.register(SigningKey)
